﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Threading;
using System.Windows;
using System.Xml.Serialization;

namespace IGPivotGrid.Samples.Controls
{
    public class SalesCollection : ObservableCollection<Sale>
    {
        public SalesCollection()
        {
            string xmlLocation = "/PivotGridFlatData;component/Assets/EN/Sales.xml";
            xmlLocation = xmlLocation.Replace("EN", Thread.CurrentThread.CurrentUICulture.Name.Substring(0, 2).ToUpper());

            var sri = Application.GetResourceStream(new Uri(xmlLocation, UriKind.RelativeOrAbsolute));
            XmlSerializer serializer = new XmlSerializer(typeof(List<Sale>));
            (serializer.Deserialize(sri.Stream) as List<Sale>).ForEach(s => this.Add(s));
        }
    }

    public class Sale
    {
        public Sale()
        {
            this.Product = new Product();
            this.Seller = new Seller();
        }
        public Product Product { get; set; }
        public Seller Seller { get; set; }
        public DateTime Date { get; set; }

        public double Value { get; set; }
        public string Quarter { get; set; }
         

        public string City
        {
            get { return Seller.City; }
            set { Seller.City = value; }
        }

        public int NumberOfUnits { get; set; }

        public double AmountOfSale
        {
            get { return UnitPrice * NumberOfUnits; }
            set { Product.UnitPrice = value / NumberOfUnits; }
        }

        public double UnitPrice
        {
            get { return Product.UnitPrice; }
            set { Product.UnitPrice = value; }
        }
    }

    public class Seller
    {
        public string Name { get; set; }
        public string City { get; set; }
    }

    public class Product
    {
        public string Name { get; set; }
        public double UnitPrice { get; set; }
    }
}

